@@ -1,19 +1,20 @@
+
 /*! \file
 Copyright (c) 2003, The Regents of the University of California, through
-Lawrence Berkeley National Laboratory (subject to receipt of any required 
-approvals from U.S. Dept. of Energy) 
+Lawrence Berkeley National Laboratory (subject to receipt of any required
+approvals from U.S. Dept. of Energy)
 
-All rights reserved. 
+All rights reserved.
 
 The source code is distributed under BSD license, see the file License.txt
 at the top-level directory.
 */
 
-/*! @file 
+/*! @file
  * \brief Driver program for testing PZGSSVX.
  *
  * <pre>
- * -- Distributed SuperLU routine (version 5.2) --
+ * -- Distributed SuperLU routine (version 9.0) --
  * Lawrence Berkeley National Lab, Univ. of California Berkeley.
  * September 30, 2017
  * </pre>
@@ -34,8 +35,10 @@ at the top-level directory.
 #include "superlu_dist_config.h"
 #include "superlu_zdefs.h"
 
+#define _XOPEN_SOURCE // C99 standard extension to support putenv()
+
 #define NTESTS 1 /*5*/      /* Number of test types */
-#define NTRAN  2    
+#define NTRAN  2
 #define THRESH 20.0
 #define FMT1   "%10s:n=%d, test(%d)=%12.5g\n"
 #define	FMT2   "%10s:fact=%4d, DiagScale=%d, n=%d, imat=%d, test(%d)=%12.5g, berr=%12.5g\n"
@@ -99,9 +102,9 @@ int main(int argc, char *argv[])
  * Purpose
  * =======
  *
- * PZTEST is the main test program for the DOUBLE COMPLEX linear 
+ * PZTEST is the main test program for the DOUBLE COMPLEX linear
  * equation driver routines PZGSSVX.
- * 
+ *
  * The program is invoked by a shell script file -- dtest.csh.
  * The output from the tests are written into a file -- dtest.out.
  */
@@ -119,13 +122,11 @@ int main(int argc, char *argv[])
     doublecomplex   *b, *bsave, *xtrue, *solx;
     int    i, j, m, n, izero = 0;
     int    nprow, npcol;
-    int    iam, info, ldb, ldx, nrhs;
-    int_t  iinfo;
+    int    iam, info, ldb, ldx, nrhs, iinfo;
     char     **cpp, c;
     FILE *fp, *fopen();
     char matrix_type[8], equed[1];
-    int  relax, maxsuper=sp_ienv_dist(3), fill_ratio=sp_ienv_dist(6),
-         min_gemm_gpu_offload=0;
+    int  relax, maxsuper, fill_ratio, min_gemm_gpu_offload=0;
     int    equil, ifact, nfact, iequil, iequed, prefact, notfactored, diaginv;
     int    nt, nrun=0, nfail=0, nerrs=0, imat, fimat=0;
     int    nimat=1;  /* Currently only test a sparse matrix read from a file. */
@@ -151,12 +152,12 @@ int main(int argc, char *argv[])
 		       &fill_ratio, &min_gemm_gpu_offload, &nrhs, &fp);
 
     /* ------------------------------------------------------------
-       INITIALIZE MPI ENVIRONMENT. 
+       INITIALIZE MPI ENVIRONMENT.
        ------------------------------------------------------------*/
     MPI_Init( &argc, &argv );
 
     /* ------------------------------------------------------------
-       INITIALIZE THE SUPERLU PROCESS GRID. 
+       INITIALIZE THE SUPERLU PROCESS GRID.
        ------------------------------------------------------------*/
     superlu_gridinit(MPI_COMM_WORLD, nprow, npcol, &grid);
 
@@ -175,21 +176,24 @@ int main(int argc, char *argv[])
     /* Set the default input options. */
     set_default_options_dist(&options);
     options.PrintStat = NO;
-	
+
     if (!iam) {
 	print_sp_ienv_dist(&options);
 	print_options_dist(&options);
 	fflush(stdout);
     }
 
+    maxsuper = sp_ienv_dist(3, &options);
+    fill_ratio = sp_ienv_dist(6, &options);
+
     if ( !(berr = doubleMalloc_dist(nrhs)) )
 	ABORT("Malloc fails for berr[].");
-	
+
     /* Loop through all the input options. */
     for (imat = fimat; imat < nimat; ++imat) { /* All matrix types */
 	//if (!iam) printf("imat loop ... %d\n", imat);
 	/* ------------------------------------------------------------
-	   GET THE MATRIX FROM FILE AND SETUP THE RIGHT HAND SIDE. 
+	   GET THE MATRIX FROM FILE AND SETUP THE RIGHT HAND SIDE.
 	   ------------------------------------------------------------*/
 	zcreate_matrix(&A, nrhs, &b, &ldb, &xtrue, &ldx, fp, &grid);
 
@@ -226,7 +230,7 @@ int main(int argc, char *argv[])
 		fact = facts[ifact];
 		options.Fact = fact;
 		//if (!iam) printf("ifact loop ... %d\n", ifact);
-#ifdef SLU_HAVE_LAPACK 
+#ifdef SLU_HAVE_LAPACK
 	        for (diaginv = 0; diaginv < 2; ++diaginv) {
 #endif
 		    for (equil = 0; equil < 2; ++equil) {
@@ -251,7 +255,7 @@ int main(int argc, char *argv[])
 
 			    R = (double *) SUPERLU_MALLOC(m*sizeof(double));
 			    C = (double *) SUPERLU_MALLOC(n*sizeof(double));
-			
+
 			    /* Later call to PZGSSVX only needs to solve. */
                             if ( equil || iequed ) {
 			        /* Compute row and column scale factors to
@@ -279,12 +283,12 @@ int main(int argc, char *argv[])
 				      ScalePermstruct.C = C;
 				   }
 			        }
-			
+
 			        /* Equilibrate the matrix. */
 			    	pzlaqgs(&A, R, C, rowcnd, colcnd, amax, equed);
 			    	// printf("after pdlaqgs: *equed %c\n", *equed);
 
-			    	/* Not equilibrate anymore when calling 
+			    	/* Not equilibrate anymore when calling
 				   PDGSSVX, so, no malloc/free {R,C}
 				   inside PDGSSVX. */
 			    	options.Equil = NO;
@@ -292,14 +296,14 @@ int main(int argc, char *argv[])
 		    	} /* end if prefact */
 
 		        if ( prefact ) { /* Need a first factor */
-			
+
 			    /* Save Fact option. */
 		            fact = options.Fact;
 			    options.Fact = DOFACT;
 
 			    /* Initialize the statistics variables. */
 			    PStatInit(&stat);
-	
+
 			    int nrhs1 = 0; /* Only performs factorization */
 			    pzgssvx(&options, &A, &ScalePermstruct, b,
                                 ldb, nrhs1, &grid, &LUstruct, &SOLVEstruct,
@@ -334,20 +338,20 @@ int main(int argc, char *argv[])
 			    /* Restore the matrix A. */
 			    zCopy_CompRowLoc_NoAllocation(&Asave, &A);
 			    if (fact == SamePattern_SameRowPerm && iam == 0) {
-                                /* Perturb the 1st diagonal of the matrix 
+                                /* Perturb the 1st diagonal of the matrix
                                    to larger value, so to have a different A. */
                                 (doublecomplex *) Astore->nzval)[0].r += 1.0e-12; //1.0e-8;
                                 (doublecomplex *) Astore->nzval)[0].i += 1.0e-12; //1.0e-8;
                              }
 
-		        } 
+		        }
 
 		        /* Set the right-hand side. */
 		        zCopy_Dense_Matrix_dist(m_loc, nrhs, bsave, ldb, b, ldb);
 
 		        PStatInit(&stat);
 
-		    /*if ( !iam ) printf("\ttest pdgssvx: nrun %d, iequed %d, equil %d, fact %d\n", 
+		    /*if ( !iam ) printf("\ttest pdgssvx: nrun %d, iequed %d, equil %d, fact %d\n",
 		      nrun, iequed, equil, options.Fact);*/
 		        /* Testing PDGSSVX: solve and compute the error bounds. */
 		        pzgssvx(&options, &A, &ScalePermstruct, b, ldb, nrhs,
@@ -357,7 +361,7 @@ int main(int argc, char *argv[])
 		        PStatFree(&stat);
 #if 0
 		        pdinf_norm_error(iam, ((NRformat_loc *)A.Store)->m_loc,
-				     nrhs, b, ldb, xtrue, ldx, &grid);
+				     nrhs, b, ldb, xtrue, ldx, grid.comm);
 #endif
 		        if ( info ) {
 			    printf(FMT3, "pzgssvx",info,izero,n,nrhs,imat,nfail);
@@ -369,13 +373,13 @@ int main(int argc, char *argv[])
 			    solx = b;
 			    pzcompute_resid(m, n, nrhs, &A, solx, ldx,
                                         bsave, ldb, &grid, &SOLVEstruct, &result[0]);
-			
+
 #if 0  /* how to get RCOND? */
 			/* Check solution accuracy from generated exact solution. */
 			    dgst04(n, nrhs, solx, ldx, xact, ldx, rcond,
 					  &result[2]);
 			    pdinf_norm_error(iam, ((NRformat_loc *)A.Store)->m_loc,
-					 nrhs, b, ldb, xtrue, ldx, &grid);
+					 nrhs, b, ldb, xtrue, ldx, grid.comm);
 #endif
 
 			    /* Print information about the tests that did
@@ -383,7 +387,7 @@ int main(int argc, char *argv[])
 			    int k1 = 0;
 			    for (i = k1; i < NTESTS; ++i) {
 			        if ( result[i] >= THRESH ) {
-				    printf(FMT2, "pzgssvx", options.Fact, 
+				    printf(FMT2, "pzgssvx", options.Fact,
 				       ScalePermstruct.DiagScale,
 				       n, imat, i, result[i], berr[0]);
 				    ++nfail;
@@ -391,7 +395,7 @@ int main(int argc, char *argv[])
 			    }
 			    nrun += NTESTS;
 		        } /* end else .. info == 0 */
-		   
+
 		        /* -------------------------------------------------
 		           Deallocate storage associated with {L,U}.
 		           ------------------------------------------------- */
@@ -409,13 +413,13 @@ int main(int argc, char *argv[])
 
 		    } /* end for equil ... */
 
-#ifdef SLU_HAVE_LAPACK 
+#ifdef SLU_HAVE_LAPACK
                 } /* end for diaginv ... */
 #endif
 	    } /* end for ifact ... */
-		
+
 	} /* end for iequed ... */
-	
+
 	/* ------------------------------------------------------------
 	   DEALLOCATE STORAGE.
 	   ------------------------------------------------------------*/
@@ -450,7 +454,7 @@ out:
 
 }
 
-/*  
+/*
  * Parse command line options to get various input parameters.
  */
 static void
@@ -490,43 +494,43 @@ parse_command_line(int argc, char *argv[], int *nprow, int *npcol,
 #ifdef _MSC_VER
 #define putenv _putenv
 #endif
-	  case 'x': // c = atoi(optarg); 
+	  case 'x': // c = atoi(optarg);
 	            // sprintf(str, "%d", c);
 	            // setenv("NREL", str, 1);
-		    xenvstr = (char*) malloc((6+strlen(optarg))*sizeof(char));
-		    strcpy(xenvstr, "NREL=");
+		    xenvstr = (char*) malloc((20+strlen(optarg))*sizeof(char));
+		    strcpy(xenvstr, "SUPERLU_RELAX=");
 		    strcat(xenvstr, optarg);
 		    putenv(xenvstr);
 	            //printf("Reset relax env. variable to %d\n", c);
 	            break;
-	  case 'm': // c = atoi(optarg); 
+	  case 'm': // c = atoi(optarg);
 	            // sprintf(str, "%d", c);
 		    // setenv("NSUP", str, 1);
-		    menvstr = (char*) malloc((6+strlen(optarg))*sizeof(char));
-		    strcpy(menvstr, "NSUP=");
+		    menvstr = (char*) malloc((20+strlen(optarg))*sizeof(char));
+		    strcpy(menvstr, "SUPERLU_MAXSUP=");
 		    strcat(menvstr, optarg);
 		    putenv(menvstr);
 		    //printf("Reset maxsuper env. variable to %d\n", c);
 	            break;
-	  case 'b': // c = atoi(optarg); 
+	  case 'b': // c = atoi(optarg);
 	            // sprintf(str, "%d", c);
 		    // setenv("FILL", str, 1);
-		    benvstr = (char*) malloc((6+strlen(optarg))*sizeof(char));
-		    strcpy(benvstr, "FILL=");
+		    benvstr = (char*) malloc((20+strlen(optarg))*sizeof(char));
+		    strcpy(benvstr, "SUPERLU_FILL=");
 		    strcat(benvstr, optarg);
 		    putenv(benvstr);
 		    //printf("Reset fill_ratio env. variable to %d\n", c);
 	            break;
-	  case 'g': // c = atoi(optarg); 
+	  case 'g': // c = atoi(optarg);
 	            // sprintf(str, "%d", c);
 		    // setenv("N_GEMM", str, 1);
-		    genvstr = (char*) malloc((8+strlen(optarg))*sizeof(char));
-		    strcpy(genvstr, "N_GEMM=");
+		    genvstr = (char*) malloc((20+strlen(optarg))*sizeof(char));
+		    strcpy(genvstr, "SUPERLU_N_GEMM=");
 		    strcat(genvstr, optarg);
 		    putenv(genvstr);
 		    //printf("Reset min_gemm_gpu_offload env. variable to %d\n", c);
 	            break;
-	  case 's': *nrhs = atoi(optarg); 
+	  case 's': *nrhs = atoi(optarg);
 	            break;
           case 'f':
                     if ( !(*fp = fopen(optarg, "r")) ) {
